@@ -5,7 +5,8 @@
 #define warn(fmt, args...) \
 	fprintf(stderr, _("%s%s%s: " fmt "\n"), RED, argv0, NORM , ## args)
 #endif
-#define warnf(fmt, args...) warn("%s%s()%s: " fmt, YELLOW, __FUNCTION__, NORM , ## args)
+#define warnf(fmt, args...) warn("%s%s()%s: " fmt, YELLOW, __func__, NORM , ## args)
+#define warnl(fmt, args...) warn("%s%i()%s: " fmt, YELLOW, __LINE__, NORM , ## args)
 #define warnp(fmt, args...) warn(fmt ": %s" , ## args , strerror(errno))
 #define warnfp(fmt, args...) warnf(fmt ": %s" , ## args , strerror(errno))
 #define _err(wfunc, fmt, args...) \
