@@ -1,15 +1,14 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/virtuals.c,v 1.16 2007/04/15 22:43:37 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/virtuals.c,v 1.18 2007/05/23 13:47:31 solar Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
  *
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/virtuals.c,v 1.16 2007/04/15 22:43:37 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/virtuals.c,v 1.18 2007/05/23 13:47:31 solar Exp $
  */
 
-
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -30,7 +29,7 @@ queue *virtuals = NULL;
 queue *del_set(char *s, queue *q, int *ok);
 queue *add_set(const char *vv, const char *ss, queue *q);
 
-void free_virtuals(queue *list);
+/* void free_virtuals(queue *list); */
 
 /* add a set to a cache */
 queue *add_set(const char *vv, const char *ss, queue *q)
