@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.140 2007/04/18 17:43:08 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.145 2007/05/14 16:35:46 solar Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -25,45 +25,8 @@
 #include <libgen.h>
 #include <limits.h>
 #include <assert.h>
-
-/* make sure our buffers are as big as they can be */
-#if PATH_MAX > _POSIX_PATH_MAX  /* _Q_PATH_MAX */
-# define _Q_PATH_MAX PATH_MAX
-#else
-# define _Q_PATH_MAX _POSIX_PATH_MAX
-#endif
-
-#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(*arr))
-
-/* http://tinderbox.dev.gentoo.org/default-linux/arm */
-/* http://tinderbox.dev.gentoo.org/default-linux/hppa */
-
-#ifdef __linux__
-#undef  URL
-#define URL "http://tinderbox.dev.gentoo.org"
-# ifdef __i386__
-#  ifdef __UCLIBC__
-#   define PORTAGE_BINHOST URL "/uclibc/i386"
-#  else
-#   ifdef __SSP__
-#    define PORTAGE_BINHOST URL "/hardened/x86"
-#   else
-#    define PORTAGE_BINHOST URL "/default-linux/x86/All"
-#   endif
-#  endif
-#  if defined(__powerpc__) && defined(__SSP__)
-#   if !defined(__UCLIBC__)
-#    define PORTAGE_BINHOST URL "/hardened/ppc"
-#   else
-#    define PORTAGE_BINHOST URL "/uclibc/ppc"
-#   endif
-#  endif
-# endif
-#endif
-
-#ifndef PORTAGE_BINHOST
-# define PORTAGE_BINHOST ""
-#endif
+#include <dlfcn.h>
+#include "main.h"
 
 /* prototypes and such */
 static char eat_file(const char *file, char *buf, const size_t bufsize);
@@ -78,19 +41,22 @@ void cleanup(void);
 int lookup_applet_idx(const char *);
 
 /* variables to control runtime behavior */
-static char exact = 0;
-static int found = 0;
-static int verbose = 0;
-static int quiet = 0;
-static char pretend = 0;
-static char reinitialize = 0;
-static char reinitialize_metacache = 0;
-static char portdir[_Q_PATH_MAX] = "/usr/portage";
-static char portarch[20] = "";
-static char portvdb[] = "var/db/pkg";
-static char portcachedir[] = "metadata/cache";
-static char portroot[_Q_PATH_MAX] = "/";
-static char config_protect[_Q_PATH_MAX] = "/etc/";
+void *dlhandle = NULL;
+char *module_name = NULL;
+char *modpath = NULL;
+char exact = 0;
+int found = 0;
+int verbose = 0;
+int quiet = 0;
+char pretend = 0;
+char reinitialize = 0;
+char reinitialize_metacache = 0;
+char portdir[_Q_PATH_MAX] = "/usr/portage";
+char portarch[20] = "";
+char portvdb[] = "var/db/pkg";
+char portcachedir[] = "metadata/cache";
+char portroot[_Q_PATH_MAX] = "/";
+char config_protect[_Q_PATH_MAX] = "/etc/";
 
 char pkgdir[512] = "/usr/portage/packages/";
 char port_tmpdir[512] = "/var/tmp/portage/";
@@ -102,25 +68,11 @@ char install_mask[BUFSIZ] = "";
 
 const char *err_noapplet = "Sorry this applet was disabled at compile time";
 
-#define qfprintf(stream, fmt, args...) do { if (!quiet) fprintf(stream, _( fmt ), ## args); } while (0)
-#define qprintf(fmt, args...) qfprintf(stdout, _( fmt ), ## args)
-
-
-
-#define _q_unused_ __attribute__((__unused__))
-
-#ifndef BUFSIZE
-# define BUFSIZE 8192
-#endif
-
 /* helper functions for showing errors */
 static const char *argv0;
 
 #ifdef EBUG
 # include <sys/resource.h>
-
-# define DBG(fmt, args...) warnf(fmt , ## args)
-# define IF_DEBUG(x) x
 void init_coredumps(void);
 void init_coredumps(void)
 {
@@ -129,9 +81,6 @@ void init_coredumps(void)
 	rl.rlim_max = RLIM_INFINITY;
 	setrlimit(RLIMIT_CORE, &rl);
 }
-#else
-# define DBG(fmt, args...)
-# define IF_DEBUG(x)
 #endif
 
 /* include common library code */
@@ -173,10 +122,11 @@ void no_colors()
 	case 'C': no_colors(); break; \
 	default: applet ## _usage(EXIT_FAILURE); break;
 
-#define GETOPT_LONG(A, a, ex) \
-	getopt_long(argc, argv, ex A ## _FLAGS, a ## _long_opts, NULL)
 /* display usage and exit */
-static void usage(int status, const char *flags, struct option const opts[],
+void usage(int status, const char *flags, struct option const opts[],
+                  const char *help[], int blabber);
+
+void usage(int status, const char *flags, struct option const opts[],
                   const char *help[], int blabber)
 {
 	unsigned long i;
@@ -191,12 +141,15 @@ static void usage(int status, const char *flags, struct option const opts[],
 					YELLOW, applets[i].name, NORM,
 					DKBLUE, applets[i].opts, NORM,
 					RED, NORM, _(applets[i].desc));
-	} else {
-		printf("%sUsage:%s %s%s%s %s%s%s %s:%s %s\n", GREEN, NORM,
-			YELLOW, applets[blabber].name, NORM,
-			DKBLUE, applets[blabber].opts, NORM,
-			RED, NORM, _(applets[blabber].desc));
+	} else if (blabber > 0) {
+			printf("%sUsage:%s %s%s%s <opts> %s%s%s %s:%s %s\n", GREEN, NORM,
+				YELLOW, applets[blabber].name, NORM,
+				DKBLUE, applets[blabber].opts, NORM,
+				RED, NORM, _(applets[blabber].desc));
 	}
+	if (module_name != NULL)
+		printf("%sLoaded module:%s\n%s%8s%s %s<args>%s\n", GREEN, NORM, YELLOW, module_name, NORM, DKBLUE, NORM);
+
 	printf("\n%sOptions:%s -[%s]\n", GREEN, NORM, flags);
 	for (i = 0; opts[i].name; ++i) {
 		assert(help[i] != NULL); /* this assert is a life saver when adding new applets. */
@@ -209,6 +162,7 @@ static void usage(int status, const char *flags, struct option const opts[],
 	}
 	exit(status);
 }
+
 static void version_barf(const char *Id)
 {
 #ifndef VERSION
@@ -948,8 +902,8 @@ char *grab_vdb_item(const char *item, const char *CATEGORY, const char *PF)
 	return buf;
 }
 
-queue *get_vdb_atoms(void);
-queue *get_vdb_atoms(void)
+queue *get_vdb_atoms(int fullcpv);
+queue *get_vdb_atoms(int fullcpv)
 {
 	int cfd, j;
 	int dfd, i;
@@ -1001,10 +955,15 @@ queue *get_vdb_atoms(void)
 				if (strcmp(buf, "0") != 0);
 			}
 
-			if (atom->PR_int)
-				snprintf(buf, sizeof(buf), "%s/%s-%s-r%i", atom->CATEGORY, atom->PN, atom->PV , atom->PR_int);
-			else
-				snprintf(buf, sizeof(buf), "%s/%s-%s", atom->CATEGORY, atom->PN, atom->PV);
+			if (fullcpv) {
+				if (atom->PR_int)
+					snprintf(buf, sizeof(buf), "%s/%s-%s-r%i", atom->CATEGORY, atom->PN, atom->PV , atom->PR_int);
+				else
+					snprintf(buf, sizeof(buf), "%s/%s-%s", atom->CATEGORY, atom->PN, atom->PV);
+			} else {
+				snprintf(buf, sizeof(buf), "%s/%s", atom->CATEGORY, atom->PN);
+			}
+
 			atom_implode(atom);
 			cpf = add_set(buf, "0", cpf);
 		}
@@ -1027,6 +986,10 @@ void cleanup()
 	reinitialize_as_needed();
 	free_sets(virtuals);
 	fclose(stderr);
+#ifndef STATIC
+	if (dlhandle != NULL)
+		dlclose(dlhandle);
+#endif
 }
 
 int main(int argc, char **argv)
@@ -1034,7 +997,7 @@ int main(int argc, char **argv)
 	IF_DEBUG(init_coredumps());
 	argv0 = argv[0];
 
-#ifdef ENABLE_NLS
+#ifdef ENABLE_NLS	/* never tested */
 	setlocale(LC_ALL, "");
 	bindtextdomain(argv0, "/usr/share/locale");
 	textdomain(argv0);
