@@ -1,32 +1,40 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/q.c,v 1.34 2007/04/18 18:20:47 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/q.c,v 1.42 2007/05/23 13:47:31 solar Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-#define Q_FLAGS "irm" COMMON_FLAGS
+#define Q_FLAGS "irmM:" COMMON_FLAGS
 static struct option const q_long_opts[] = {
 	{"install",      no_argument, NULL, 'i'},
 	{"reinitialize", no_argument, NULL, 'r'},
 	{"metacache",    no_argument, NULL, 'm'},
+	{"modpath",       a_argument, NULL, 'M'},
 	COMMON_LONG_OPTS
 };
 static const char *q_opts_help[] = {
 	"Install symlinks for applets",
 	"Reinitialize ebuild cache",
 	"Reinitialize metadata cache",
+	"Module path",
 	COMMON_OPTS_HELP
 };
-static const char q_rcsid[] = "$Id: q.c,v 1.34 2007/04/18 18:20:47 vapier Exp $";
+static const char q_rcsid[] = "$Id: q.c,v 1.42 2007/05/23 13:47:31 solar Exp $";
 #define q_usage(ret) usage(ret, Q_FLAGS, q_long_opts, q_opts_help, lookup_applet_idx("q"))
 
+#ifndef STATIC
+APPLET lookup_dl_applet(char *applet);
+#endif
 
 APPLET lookup_applet(char *applet);
 APPLET lookup_applet(char *applet)
 {
+#ifndef STATIC
+	APPLET dl_func;
+#endif
 	unsigned int i;
 
 	if (strlen(applet) < 1)
@@ -50,12 +58,73 @@ APPLET lookup_applet(char *applet)
 			return applets[i].func;
 		}
 	}
-
+#ifndef STATIC
+	if ((dl_func = lookup_dl_applet(applet)) != NULL)
+		return dl_func;
+	if (*applet == 'q')
+		if ((dl_func = lookup_dl_applet(applet+1)) != NULL)
+			return dl_func;
+#endif
 	/* still nothing ?  those bastards ... */
 	warn("Unknown applet '%s'", applet);
 	return NULL;
 }
 
+#ifndef STATIC
+APPLET lookup_dl_applet(char *applet) {
+	char *ptr = NULL;
+	APPLET iptr;
+	FILE *fp;
+	char buf[_Q_PATH_MAX];
+
+	if (dlhandle != NULL)
+		return NULL;
+
+	/* modpath can be passed in as a global option */
+	if (modpath == NULL) {
+		DBG("opening /etc/q.conf");
+		if ((fp = fopen("/etc/q.conf", "r")) == NULL)
+			return NULL;
+
+		while ((fgets(buf, sizeof(buf), fp)) != NULL) {
+			rmspace(buf);
+			remove_extra_space(buf);
+			if ((strncmp(buf, "modpath=", 8)) == 0)
+				modpath = &buf[8];
+		}
+		fclose(fp);
+	}
+	if (modpath == NULL)
+		return NULL;
+
+	if (!strlen(modpath))
+		return NULL;
+
+	DBG("module path set to %s", modpath);
+
+	/* fill in the path, check that object exists and open it */
+	xasprintf(&ptr, "%s/lib%s.so", modpath, applet);
+	if ((access(ptr, X_OK)) != 0) {
+		DBG("file does not exist or we dont have perms to mmap %s", ptr);
+		free(ptr);
+		return NULL;
+	}
+	if ((dlhandle = dlopen(ptr, RTLD_LAZY | RTLD_GLOBAL)) == NULL) {
+		DBG("unable to load %s : %s", ptr, dlerror());
+		free(ptr);
+		return NULL;
+	}
+	free(ptr);
+
+	xasprintf(&ptr, "%s_main", applet);
+	DBG("symbol lookup %s", ptr);
+	iptr = (APPLET) dlsym(dlhandle, ptr);
+	free(ptr);
+	module_name = applet;
+	return iptr;
+}
+#endif
+
 int lookup_applet_idx(const char *applet)
 {
 	unsigned int i;
@@ -87,6 +156,7 @@ int q_main(int argc, char **argv)
 	while ((i = GETOPT_LONG(Q, q, "+")) != -1) {
 		switch (i) {
 		COMMON_GETOPTS_CASES(q)
+		case 'M': modpath = optarg; break;
 		case 'm': reinitialize_metacache = 1; break;
 		case 'r': reinitialize = 1; break;
 		case 'i': {
