@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qlop.c,v 1.36 2007/01/10 03:42:19 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qlop.c,v 1.40 2007/05/23 03:22:31 solar Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -23,9 +23,7 @@
 
 #define QLOP_DEFAULT_LOGFILE "/var/log/emerge.log"
 
-
-
-#define QLOP_FLAGS "gtluscf:F:H" COMMON_FLAGS
+#define QLOP_FLAGS "gtHluscf:" COMMON_FLAGS
 static struct option const qlop_long_opts[] = {
 	{"gauge",     no_argument, NULL, 'g'},
 	{"time",      no_argument, NULL, 't'},
@@ -48,7 +46,7 @@ static const char *qlop_opts_help[] = {
 	"Read emerge logfile instead of " QLOP_DEFAULT_LOGFILE,
 	COMMON_OPTS_HELP
 };
-static const char qlop_rcsid[] = "$Id: qlop.c,v 1.36 2007/01/10 03:42:19 vapier Exp $";
+static const char qlop_rcsid[] = "$Id: qlop.c,v 1.40 2007/05/23 03:22:31 solar Exp $";
 #define qlop_usage(ret) usage(ret, QLOP_FLAGS, qlop_long_opts, qlop_opts_help, lookup_applet_idx("qlop"))
 
 #define QLOP_LIST    0x01
