@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.65 2007/04/18 18:47:18 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.70 2007/05/23 03:22:31 solar Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -53,7 +53,7 @@ static const char *qmerge_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.65 2007/04/18 18:47:18 solar Exp $";
+static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.70 2007/05/23 03:22:31 solar Exp $";
 #define qmerge_usage(ret) usage(ret, QMERGE_FLAGS, qmerge_long_opts, qmerge_opts_help, lookup_applet_idx("qmerge"))
 
 char search_pkgs = 0;
@@ -204,14 +204,8 @@ void qmerge_initialize(const char *Packages)
 	if (access("/bin/sh", X_OK) != 0)
 		err("/bin/sh must be installed");
 
-	if (pkgdir[0] == '/') {
-		int len = strlen(pkgdir);
-		if (len > 5) {
-			if ((strcmp(&pkgdir[len-4], "/All")) != 0)
-				strncat(pkgdir, "/All", sizeof(pkgdir));
-		} else
-			errf("PKGDIR='%s' is to short to be valid", pkgdir);
-	}
+	if (pkgdir[0] != '/')
+		errf("PKGDIR='%s' does not appear to be valid", pkgdir);
 
 	if (!binhost[0])
 		errf("PORTAGE_BINHOST= does not appear to be valid");
@@ -219,7 +213,6 @@ void qmerge_initialize(const char *Packages)
 	if (!search_pkgs && !pretend) {
 		if ((access(pkgdir, R_OK|W_OK|X_OK)) != 0)
 			errf("Wrong perms on PKGDIR='%s'", pkgdir);
-
 		mkdir(port_tmpdir, 0755);
 	}
 
@@ -342,7 +335,6 @@ void install_mask_pwd(int iargc, char **iargv, const struct stat st)
 	}
 }
 
-
 char *atom2str(depend_atom *atom, char *buf, size_t size);
 char *atom2str(depend_atom *atom, char *buf, size_t size)
 {
@@ -411,8 +403,11 @@ char qprint_tree_node(int level, depend_atom *atom, struct pkg_t *pkg)
 	}
 	printf("[%s] ", buf);
 	for (i = 0; i < level; i++) putchar(' ');
-	printf("%s%s/%s%s %s%s%s%s%s%s\n", DKGREEN, pkg->CATEGORY, pkg->PF, NORM,
-		install_ver, strlen(pkg->USE)>0 ? "(" : "", RED, pkg->USE, NORM, strlen(pkg->USE)>0 ? ")" : "");
+	if (verbose)
+		printf("%s%s/%s%s %s%s%s%s%s%s\n", DKGREEN, pkg->CATEGORY, pkg->PF, NORM,
+			install_ver, strlen(pkg->USE) > 0 ? "(" : "", RED, pkg->USE, NORM, strlen(pkg->USE) > 0 ? ")" : "");
+	else
+		printf("%s%s/%s%s\n", DKGREEN, pkg->CATEGORY, pkg->PF, NORM);
 	return c;
 }
 
@@ -549,7 +544,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 
 	/* split the tbz and xpak data */
 	snprintf(tarball, sizeof(tarball), "%s.tbz2", pkg->PF);
-	snprintf(buf, sizeof(buf), "%s/%s", pkgdir, tarball);
+	snprintf(buf, sizeof(buf), "%s/%s/%s", pkgdir, pkg->CATEGORY, tarball);
 	unlink(tarball);
 	symlink(buf, tarball);
 	mkdir("vdb", 0755);
@@ -824,15 +819,6 @@ int pkg_unmerge(char *cat, char *pkgname)
 	llist_char *dirs = NULL;
 
 	if ((strchr(pkgname, ' ') != NULL) || (strchr(cat, ' ') != NULL)) {
-#if 0
-		queue *vdb = NULL;
-		free_sets(vdb);
-		vdb = NULL;
-
-		vdb = get_vdb_atoms(vdb);
-		free_sets(vdb);
-		vdb = NULL;
-#endif
 		qfprintf(stderr, "%s!!!%s '%s' '%s' (ambiguous name) specify fully-qualified pkgs\n", RED, NORM, cat, pkgname);
 		qfprintf(stderr, "%s!!!%s %s/%s (ambiguous name) specify fully-qualified pkgs\n", RED, NORM, cat, pkgname);
 		/* qfprintf(stderr, "%s!!!%s %s %s (ambiguous name) specify fully-qualified pkgs\n", RED, NORM, pkgname); */
@@ -971,14 +957,15 @@ int match_pkg(const char *name, struct pkg_t *pkg)
 
 int pkg_verify_checksums(char *fname, struct pkg_t *pkg, depend_atom *atom, int strict, int display)
 {
-	char *hash;
+	char *hash = NULL;
 	int ret = 0;
 
 	if (nomd5)
 		return ret;
 
 	if (pkg->MD5[0]) {
-		hash = (char*) hash_file(fname, HASH_MD5);
+		if ((hash = (char*) hash_file(fname, HASH_MD5)) == NULL)
+			errf("hash is NULL for %s", fname);
 		if (strcmp(hash, pkg->MD5) == 0) {
 			if (display)
 				qprintf("MD5:  [%sOK%s] %s %s/%s\n", GREEN, NORM, hash, atom->CATEGORY, pkg->PF);
@@ -1013,17 +1000,20 @@ int pkg_verify_checksums(char *fname, struct pkg_t *pkg, depend_atom *atom, int
 void pkg_fetch(int argc, char **argv, struct pkg_t *pkg)
 {
 	depend_atom *atom;
-	char buf[255], buf2[255];
+	char buf[255], str[255];
 	int i;
 
+	memset(buf, 0, sizeof(buf));
+	memset(str, 0, sizeof(str));
+
 	snprintf(buf, sizeof(buf), "%s/%s", pkg->CATEGORY, pkg->PF);
 	if ((atom = atom_explode(buf)) == NULL)
 		errf("%s/%s is not a valid atom", pkg->CATEGORY, pkg->PF);
 
-	snprintf(buf2, sizeof(buf2), "%s/%s", pkg->CATEGORY, atom->PN);
+	snprintf(str, sizeof(str), "%s/%s", pkgdir, pkg->CATEGORY);
+	mkdir(str, 0755);
 
 	for (i = 1; i < argc; i++) {
-
 		if (argv[i][0] == '-')
 			continue;
 
@@ -1041,12 +1031,13 @@ void pkg_fetch(int argc, char **argv, struct pkg_t *pkg)
 		}
 
 		/* check to see if file exists and it's checksum matches */
-		snprintf(buf, sizeof(buf), "%s/%s.tbz2", pkgdir, pkg->PF);
+		snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, pkg->CATEGORY, pkg->PF);
 		unlink_empty(buf);
-		if ((force_download) && (access(buf, R_OK) == 0) && ((pkg->SHA1[0]) || (pkg->MD5[0])))
+
+		if ((force_download) && (access(buf, R_OK) == 0) && ((pkg->SHA1[0]) || (pkg->MD5[0]))) {
 			if ((pkg_verify_checksums(buf, pkg, atom, 0, 0)) != 0)
 				unlink(buf);
-
+		}
 		if (access(buf, R_OK) == 0) {
 			if ((!pkg->SHA1[0]) && (!pkg->MD5[0])) {
 				warn("No checksum data for %s", buf);
@@ -1062,11 +1053,17 @@ void pkg_fetch(int argc, char **argv, struct pkg_t *pkg)
 			printf("Fetching %s/%s.tbz2\n", atom->CATEGORY, pkg->PF);
 
 		/* fetch the package */
-		snprintf(buf, sizeof(buf), "%s.tbz2", pkg->PF);
-		fetch(pkgdir, buf);
+		/* Check CATEGORY first */
+		snprintf(buf, sizeof(buf), "%s/%s.tbz2", atom->CATEGORY, pkg->PF);
+		fetch(str, buf);
 
+		snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
+		if (access(buf, R_OK) != 0) {
+			snprintf(buf, sizeof(buf), "%s.tbz2", pkg->PF);
+			fetch(str, buf);
+		}
 		/* verify the pkg exists now. unlink if zero bytes */
-		snprintf(buf, sizeof(buf), "%s/%s.tbz2", pkgdir, pkg->PF);
+		snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
 		unlink_empty(buf);
 
 		if (access(buf, R_OK) != 0) {
@@ -1170,6 +1167,7 @@ struct pkg_t *grab_binpkg_info(const char *name)
 	char buf[BUFSIZ];
 	char value[BUFSIZ];
 	char *p;
+	depend_atom *atom;
 
 	struct pkg_t *pkg = xmalloc(sizeof(struct pkg_t));
 	struct pkg_t *rpkg = xmalloc(sizeof(struct pkg_t));
@@ -1192,8 +1190,6 @@ struct pkg_t *grab_binpkg_info(const char *name)
 
 				snprintf(buf, sizeof(buf), "%s/%s", pkg->CATEGORY, pkg->PF);
 				if (strstr(buf, name) != NULL) {
-					depend_atom *atom;
-
 					if (!best_match[0])
 						strncpy(best_match, buf, sizeof(best_match));
 
@@ -1244,6 +1240,16 @@ struct pkg_t *grab_binpkg_info(const char *name)
 			if ((strcmp(buf, "CATEGORY:")) == 0)
 				strncpy(pkg->CATEGORY, value, sizeof(Pkg.CATEGORY));
 
+			if ((strcmp(buf, "CPV:")) == 0) {
+				if ((atom = atom_explode(value)) != NULL) {
+					snprintf(buf, sizeof(buf), "%s-%s", atom->PN, atom->PV);
+					if (atom->PR_int)
+						snprintf(buf, sizeof(buf), "%s-%s-r%i", atom->PN, atom->PV, atom->PR_int);
+					strncpy(pkg->PF, buf, sizeof(Pkg.PF));
+					strncpy(pkg->CATEGORY, atom->CATEGORY, sizeof(Pkg.CATEGORY));
+					atom_implode(atom);
+				}
+			}
 			if ((strcmp(buf, "SLOT:")) == 0)
 				strncpy(pkg->SLOT, value, sizeof(Pkg.SLOT));
 			if ((strcmp(buf, "USE:")) == 0)
@@ -1325,6 +1331,17 @@ char *find_binpkg(const char *name)
 		strncpy(value, p, sizeof(value));
 
 		if (*buf) {
+			if ((strcmp(buf, "CPV:")) == 0) {
+				depend_atom *atom;
+				if ((atom = atom_explode(value)) != NULL) {
+					snprintf(buf, sizeof(buf), "%s-%s", atom->PN, atom->PV);
+					if (atom->PR_int)
+						snprintf(buf, sizeof(buf), "%s-%s-r%i", atom->PN, atom->PV, atom->PR_int);
+					strncpy(PF, buf, sizeof(PF));
+					strncpy(CATEGORY, atom->CATEGORY, sizeof(CATEGORY));
+					atom_implode(atom);
+				}
+			}
 			if ((strcmp(buf, "PF:")) == 0)
 				strncpy(PF, value, sizeof(PF));
 			if ((strcmp(buf, "CATEGORY:")) == 0)
@@ -1407,6 +1424,17 @@ int parse_packages(const char *Packages, int argc, char **argv)
 				break;
 			case 'C':
 				if ((strcmp(buf, "CATEGORY:")) == 0) strncpy(Pkg.CATEGORY, value, sizeof(Pkg.CATEGORY));
+				if ((strcmp(buf, "CPV:")) == 0) {
+					depend_atom *atom;
+					if ((atom = atom_explode(value)) != NULL) {
+						snprintf(buf, sizeof(buf), "%s-%s", atom->PN, atom->PV);
+						if (atom->PR_int)
+							snprintf(buf, sizeof(buf), "%s-%s-r%i", atom->PN, atom->PV, atom->PR_int);
+						strncpy(Pkg.PF, buf, sizeof(Pkg.PF));
+						strncpy(Pkg.CATEGORY, atom->CATEGORY, sizeof(Pkg.CATEGORY));
+						atom_implode(atom);
+					}
+				}
 				break;
 			case 'D':
 				if ((strcmp(buf, "DESC:")) == 0) strncpy(Pkg.DESC, value, sizeof(Pkg.DESC));
@@ -1420,6 +1448,45 @@ int parse_packages(const char *Packages, int argc, char **argv)
 	return 0;
 }
 
+queue *get_world(void);
+queue *get_world(void) {
+	FILE *fp;
+	char buf[BUFSIZ];
+	queue *world = NULL;
+	char *fname = (char *) "/var/lib/portage/world";
+
+	if ((fp = fopen(fname, "r")) == NULL) {
+		warn("fopen(\"%s\", \"r\"); = -1 (%s)", fname, strerror(errno));
+		return NULL;
+	}
+
+	while ((fgets(buf, sizeof(buf), fp)) != NULL) {
+		char *p;
+		char *slot = (char *) "0";
+
+		rmspace(buf);
+
+		if ((p = strchr(buf, ':')) != NULL) {
+			*p = 0;
+			slot = p + 1;
+		}
+		world = add_set(buf, slot, world);
+	}
+	fclose(fp);
+	return world;
+}
+
+queue *qmerge_load_set(char *, queue *);
+queue *qmerge_load_set(char *buf, queue *set) {
+	if (set != NULL)
+		return set;
+	if ((strcmp(buf, "world") == 0))
+		return get_world();
+	if ((strcmp(buf, "all") == 0))
+		return get_vdb_atoms(0);
+	return NULL;
+}
+
 int qmerge_main(int argc, char **argv)
 {
 	int i;
@@ -1465,13 +1532,17 @@ int qmerge_main(int argc, char **argv)
 
 	qmerge_initialize(Packages);
 
-	if (argc > 1) {
-		queue *world;
-		for (i = 0; i < argc; i++) {
+	if (optind < argc) {
+		queue *world = NULL;
+		int ind = optind;
+		while (ind < argc) {
 			size_t size = 0;
-			if ((strcmp(argv[i], "world") != 0))
+			i = ind;
+			ind++;
+
+			if ((world = qmerge_load_set(argv[i], world)) == NULL)
 				continue;
-			world = get_vdb_atoms();
+
 			if (world != NULL) {
 				queue *ll;
 				char *ptr;
@@ -1496,6 +1567,10 @@ int qmerge_main(int argc, char **argv)
 				/* this will leak mem */
 				/* follow_rdepends = 0; */
 				makeargv(ptr, &ARGC, &ARGV);
+
+				free(ptr);
+				free_sets(world);
+				world = NULL;
 			}
 		}
 	}
